package
{
    import loom.Application;

    import loom2d.display.Image;
    import loom2d.display.StageScaleMode;
    import loom2d.display.Sprite;
    import loom2d.display.Stage;
    import loom2d.display.Quad;

    import loom2d.textures.Texture;

    import loom2d.ui.SimpleLabel;

    import loom2d.math.Point;
    import loom2d.math.Rectangle;
    import loom2d.math.Color;

    import loom2d.events.KeyboardEvent;

    import loom.gameframework.TimeManager;
    import loom.gameframework.ITicked;

    import loom.platform.LoomKey;

    public static class theInput
    {
        public static var Up:Boolean;
        public static var Down:Boolean;
        public static var Left:Boolean;
        public static var Right:Boolean;

        public static var Space:Boolean;

        public static function GetAndClearSpace() : Boolean
        {
            var pressed:Boolean = Space;
            Space = false;
            return pressed;
        }

    }

    public static class theUtilities
    {
        public static var time:TimeManager;
        public static var theGame:LD48Game;

        public static function theUtilities()
        {
            time = new TimeManager();
        }

        public static function GetVectorFromAngle( ang:float ):Point
        {
            var facing:Point = Point.polar(1.0, ang);
            return new Point( facing.y, facing.x * -1);
        }

        public static function GetAngleFromVector( vec:Point ):float
        {
            var retValue:float = ( -( -180 + (Math.atan2(vec.x, vec.y) * 180 / Math.PI)));
            return retValue;
        }

        public static function Dot2D( a:Point, b:Point):float
        {
            return (a.x * b.x) + (a.y * b.y);
        }
        

    }

    public static class theCollision
    {
        public static var walls:Vector.<shepActor>;
        public static var projectiles:Vector.<shepBullet>;
        public static var bodies:Vector.<shepCharacter>;
        public static var collectables:Vector.<shepCollectable>;
        public static var particleQuads:Vector.<shepParticleQuad>;

        public static function theCollision()
        {
            walls = new Vector.<shepActor>;
            projectiles = new Vector.<shepBullet>;
            bodies = new Vector.<shepCharacter>;
            collectables = new Vector.<shepActor>;
            particleQuads = new Vector.<shepParticleQuad>;            
        }
    }


    public class LD48Game extends Application
    {

        var bExitGame = new Boolean;
        public var playerShip:shepPlayerShip;

        public var debugLabel1:SimpleLabel;

        public var currentCameraPosition:Point;

        public var playerHealthBar:shepStatusBar;

        public var gameState:int;


        public var titleCard:Image;
        public var howToPlayCard:Image;

        public var victoryCard:Image;
        public var defeatCard:Image;


        public var flashquad:Quad;

        public var bShouldFlashUp:boolean;
        public var bShouldFlashDown:boolean;
        public var flashTimer:float;
        public var flashColor:Color;

        public var lastUpdateTime:float;
        public var deltaMS:float;

        public var playTime:float;

        public var bVictoryAfterFlash:boolean;
        public var bGGAfterFlash:boolean;

        public var bDontFuckWithStars:boolean;

        override public function run():void
        {
            // Comment out this line to turn off automatic scaling.
            stage.scaleMode = StageScaleMode.LETTERBOX;
            stage.depthSort = true;

            if( !bDontFuckWithStars)
            {
                for(var t:int; t < 50; t++)
                {
                    var star:shepStar = new shepStar();
                    star.size = 2 + Math.random() * 6;
                    star.speedMultiplier = Math.random();

                    star.screenPos.x = Math.random() * stage.stageWidth;
                    star.screenPos.y = Math.random() * stage.stageHeight;

                    RegisterShepObject(star);
                }
            }
            bDontFuckWithStars = true;

            theUtilities.theGame = this;

            // Setup anything else, like UI, or game objects.
            var bg = new Image(Texture.fromAsset("assets/backdrop.png"));
            bg.width = stage.stageWidth;
            bg.height = stage.stageHeight;
            bg.depth = -1.0;
            stage.addChild(bg);

            flashquad = new Quad(0,0, new Color(255,255,255,0).toInt(), true);

            flashquad.setVertexPosition(0,0,0);
            flashquad.setVertexPosition(1,stage.stageWidth,0);
            flashquad.setVertexPosition(2,0,stage.stageHeight);
            flashquad.setVertexPosition(3,stage.stageWidth,stage.stageHeight);

            flashquad.setVertexAlpha(0,0);
            flashquad.setVertexAlpha(1,0);
            flashquad.setVertexAlpha(2,0);
            flashquad.setVertexAlpha(3,0);

            flashquad.depth = 1.0;
            stage.addChild(flashquad);

            stage.addEventListener(KeyboardEvent.KEY_UP, OnKeyUp);
            stage.addEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);

            PrepareTitleScreen();

        }

        function TickFlash( v:float)
        {
            flashquad.setVertexAlpha(0,v);
            flashquad.setVertexAlpha(1,v);
            flashquad.setVertexAlpha(2,v);
            flashquad.setVertexAlpha(3,v);

            flashquad.setVertexColor(0,flashColor.toInt());
            flashquad.setVertexColor(1,flashColor.toInt());
            flashquad.setVertexColor(2,flashColor.toInt());
            flashquad.setVertexColor(3,flashColor.toInt());
        }

        function PrepareTitleScreen():void
        {
            titleCard= new Image(Texture.fromAsset("assets/logo.png"));
            titleCard.x = stage.stageWidth / 2 - titleCard.width / 2;
            titleCard.y =  400 - titleCard.height /2;
            titleCard.depth = 0.0;
            stage.addChild(titleCard);

        }

        function CleanTitleScreen():void
        {
            stage.removeChild(titleCard);
        }

        function PrepareHowToPlay():void
        {
            gameState = 1;
            howToPlayCard= new Image(Texture.fromAsset("assets/howtoplay.png"));
            howToPlayCard.x = stage.stageWidth / 2 - howToPlayCard.width / 2;
            howToPlayCard.y = 50;
            howToPlayCard.depth = 0.0;
            stage.addChild(howToPlayCard);
        }

        function CleanHowToPlay()
        {
            stage.removeChild(howToPlayCard);
        }

        function PrepareWinScreen():void
        {
            victoryCard= new Image(Texture.fromAsset("assets/you_win.png"));
            victoryCard.x = stage.stageWidth / 2 - victoryCard.width / 2;
            victoryCard.y =  400 - victoryCard.height /2;
            victoryCard.depth = 0.0;
            stage.addChild(victoryCard);

        }

        function CleanWinScreen():void
        {
            stage.removeChild(victoryCard);
        }

        function PrepareLoseScreen():void
        {
            defeatCard= new Image(Texture.fromAsset("assets/game_over.png"));
            defeatCard.x = stage.stageWidth / 2 - defeatCard.width / 2;
            defeatCard.y =  400 - defeatCard.height /2;
            defeatCard.depth = 0.0;
            stage.addChild(defeatCard);

        }

        function CleanLoseScreen():void
        {
            stage.removeChild(defeatCard);
        }


        function EnterPressed():void
        {
            if( gameState == 0 )
            {
                CleanTitleScreen();
                PrepareHowToPlay();
            }
            else if( gameState == 1)
            {
                CleanHowToPlay();
                PrepareStartingActors();
            }
            else if( gameState == 3)
            {
                gameState = 0;
                CleanWinScreen();
                PrepareTitleScreen();
            }
            else if( gameState == 4)
            {
                gameState = 0;
                CleanLoseScreen();
                PrepareTitleScreen();
            }
        }

        function EscapePressed():void
        {

        }

        function CleanAllGame():void
        {
            var t:int =0;
            var l:int =0;

            l=theCollision.bodies.length;
            for(t = 0;  t < l; t++)
            {
                var s:shepActor = theCollision.bodies[t];
                theUtilities.time.removeTickedObject(s);
                theUtilities.time.removeAnimatedObject(s);
                s.UnRegisterDrawElements(this);
            }

            l = theCollision.projectiles.length;
            for(t = 0;  t < l; t++)
            {
                UnRegisterShepObject(theCollision.projectiles[t]);
            }
        
            l = theCollision.collectables.length; 
            for(t = 0; t < l; t++)
            {
                UnRegisterShepObject(theCollision.collectables[t]);
            }

            for(t = 0; t < theCollision.particleQuads.length; t++)
            {
                theCollision.particleQuads[t].bActive = false;
            }

            theCollision.projectiles.clear();
            theCollision.bodies.clear();
            theCollision.collectables.clear();
            theCollision.particleQuads.clear();

            theCollision.projectiles = new Vector.<shepBullet>;
            theCollision.bodies = new Vector.<shepCharacter>;
            theCollision.collectables = new Vector.<shepActor>;
            theCollision.particleQuads = new Vector.<shepParticleQuad>; 

            UnRegisterShepObject(playerShip);
            playerShip = null;

            UnRegisterShepObject(playerHealthBar);
            playerHealthBar.UnRegisterDrawElements(this);
            playerHealthBar = null;   

            bGGAfterFlash = false;
            bVictoryAfterFlash = false;
        }

        function PrepareStartingActors():void
        {
            gameState = 2;

            playTime = 0.0;

            playerHealthBar = new shepStatusBar();
            playerHealthBar.Init( this );
            playerHealthBar.xLoc = 20;
            playerHealthBar.yLoc = 10;
            playerHealthBar.Height = stage.stageHeight * 0.05;
            playerHealthBar.Width = stage.stageWidth * 0.95;

            playerHealthBar.fullColor = new Color(255,210,0,255);
            playerHealthBar.emptyColor = new Color(10,10,128,255);
            playerHealthBar.targetValue = 0.5;
            playerHealthBar.changeSpeed = 0.2;
            RegisterShepObject(playerHealthBar);

            // Setup anything else, like UI, or game objects.
            var bg = new Image(Texture.fromAsset("assets/backdrop.png"));
            bg.width = stage.stageWidth;
            bg.height = stage.stageHeight;
            bg.depth = -1.0;
            stage.addChild(bg);

            playerShip = new shepPlayerShip();
            playerShip.Init( this );
            playerShip.SetPowerLevel(0);
            playerShip.clipRect = new Rectangle();
            playerShip.bPlayerControlled = true;
            playerShip.worldPosition.x = 300;
            playerShip.worldPosition.y = 300;
            playerShip.moveFriction = 50.0;
            playerShip.rotationFriction = 36.0;
            playerShip.faction = 1;
            RegisterShepObject(playerShip);


            var powerup = new shepCollectable();
            powerup.Init( this );
            powerup.sprite = new Image(Texture.fromAsset("assets/powerup_one.png"));
            powerup.sprite.center();
            powerup.worldPosition.x = playerShip.worldPosition.x;
            powerup.worldPosition.y = playerShip.worldPosition.y - 200;
            powerup.clipRect = new Rectangle();
            powerup.clipRect.x = powerup.worldPosition.x;
            powerup.clipRect.y = powerup.worldPosition.y;
            powerup.clipRect.width = powerup.sprite.width;
            powerup.clipRect.height = powerup.sprite.height;

            powerup.bIncreasePlayerPowerLevel = true;
            powerup.bActive = true;

            RegisterShepObject(powerup);
            theCollision.collectables.push(powerup);

            for( var pidx:int =0 ; pidx < 50; pidx++)
            {
                var bullet = new shepBullet();
                theCollision.projectiles.push(bullet);
            }

            for( var cidx:int =0 ; cidx < 80; cidx++)
            {
                var collect = new shepCollectable();
                theCollision.collectables.push(collect);
            }

            for( var pridx:int =0 ; pridx < 200; pridx++)
            {
                var pquad = new shepParticleQuad();
                theCollision.particleQuads.push(pquad);
            }


        }

        public override function onTick():void
        {
            var oldTime = lastUpdateTime;
            lastUpdateTime = theUtilities.time.virtualTime;
            deltaMS = lastUpdateTime - oldTime;
            deltaMS /= 1000.0;

            if( gameState == 2)
                playTime += deltaMS;

            if( playerShip != null )
            {
                currentCameraPosition.x = playerShip.worldPosition.x - (stage.stageWidth / 2);
                currentCameraPosition.y = playerShip.worldPosition.y - (stage.stageHeight / 2);
            }

            theMonsterSpawner.onTick();

            if( bShouldFlashUp )
            {
                flashColor = new Color(255,255,0,255);
                flashTimer -= deltaMS;

                TickFlash( 1.0 - (flashTimer/1.0));

                if( flashTimer <= 0)
                {
                    TickFlash(0);
                    bShouldFlashUp = false;

                    for(var sparkleint:int = 0; sparkleint < 20; sparkleint++)
                    {
                        SpawnQuadParticle( playerShip.worldPosition,  new Point( -400 + Math.random() * 800, -400 + Math.random() * 800), new Point(0,0), 0.5, new Color(255,255,0,255), 6.0,2.0,1.0 );
                    }

                    if( bVictoryAfterFlash )
                    {
                        CleanAllGame();
                        gameState = 3;
                        PrepareWinScreen();
                    }
                }
            }

            if( bShouldFlashDown )
            {
                flashColor = new Color(0,0,255,255);
                flashTimer -= deltaMS;

                TickFlash( flashTimer/2.0);

                if( flashTimer <= 0)
                {
                    TickFlash(0);
                    bShouldFlashDown = false;
                    if( bGGAfterFlash )
                    {
                        CleanAllGame();
                        gameState = 4;
                        PrepareLoseScreen();
                    }
                }
            }

        }

        public function StartFlashDown():void
        {
            if( bShouldFlashDown )
                return;

            bShouldFlashDown = true;
            flashTimer = 2.0;
        }

        public function StartFlashUp():void
        {
            if( bShouldFlashUp == true)
                return;
                
            bShouldFlashUp = true;
            flashTimer = 1.0;
        }

        public function GetStage():Stage
        {
            return stage;
        }

        public function RegisterShepObject( s:shepObject ):void
        {
            if( gameState > 2 )
                return;

            theUtilities.time.addTickedObject(s);
            theUtilities.time.addAnimatedObject(s);
            s.lastUpdateTime = theUtilities.time.virtualTime;
            s.RegisterDrawElements(this);
        }

        public function UnRegisterShepObject( s:shepObject ):void
        {
            theUtilities.time.removeTickedObject(s);
            theUtilities.time.removeAnimatedObject(s);
            s.UnRegisterDrawElements(this);
            theCollision.bodies.remove(s);
        }

        override public function onFrame():void
        {
            super.onFrame();
        }

        protected function OnKeyDown( event:KeyboardEvent):void
        {
            switch(event.keyCode)
            {
                case LoomKey.W:
                    theInput.Up = true;
                    break;
                case LoomKey.A:
                    theInput.Left = true;
                    break;
                case LoomKey.S:
                    theInput.Down = true;
                    break;
                case LoomKey.D:
                    theInput.Right = true;
                    break;
                case LoomKey.UP_ARROW:
                    theInput.Up = true;
                    break;
                case LoomKey.LEFT_ARROW:
                    theInput.Left = true;
                    break;
                case LoomKey.DOWN_ARROW:
                    theInput.Down = true;
                    break;
                case LoomKey.RIGHT_ARROW:
                    theInput.Right = true;
                    break;
                case LoomKey.SPACEBAR:
                    theInput.Space = true;
                    if( gameState != 2 )
                        EnterPressed();
                    break;
                case LoomKey.Z:
                    theInput.Space = true;
                    if( gameState != 2 )
                        EnterPressed();
                    break;

                case LoomKey.RETURN:
                    EnterPressed();
                    break;

                case LoomKey.ONE:
                    theInput.Space = true;
                    if( gameState != 2 )
                        EnterPressed();
                    break;
                
                case LoomKey.TWO:
                    playerShip.SetPowerLevel(2);
                    break;
                case LoomKey.THREE:
                    playerShip.SetPowerLevel(3);
                    break;
                case LoomKey.SIX:
                    playerShip.SetPowerLevel(6);
                    break;
                case LoomKey.SEVEN:
                    playerShip.SetPowerLevel(7);
                    break;
                case LoomKey.EIGHT:
                    playerShip.SetPowerLevel(8);
                    break;
                case LoomKey.NINE:
                    playerShip.SetPowerLevel(9);
                    break;

                case LoomKey.O:
                    playerShip.TakeDamage(5);
                    break;
                case LoomKey.P:
                    SpawnEnemy(9, new Point( playerShip.worldPosition.x + 400, playerShip.worldPosition.y));
                    break;
               

            }
           
        }

        protected function OnKeyUp( event:KeyboardEvent):void
        {
            switch(event.keyCode)
            {
                case LoomKey.W:
                    theInput.Up = false;
                    break;
                case LoomKey.A:
                    theInput.Left = false;
                    break;
                case LoomKey.S:
                    theInput.Down = false;
                    break;
                case LoomKey.D:
                    theInput.Right = false;
                    break;
                case LoomKey.UP_ARROW:
                    theInput.Up = false;
                    break;
                case LoomKey.LEFT_ARROW:
                    theInput.Left = false;
                    break;
                case LoomKey.DOWN_ARROW:
                    theInput.Down = false;
                    break;
                case LoomKey.RIGHT_ARROW:
                    theInput.Right = false;
                    break;
            }

        }

        public function LaunchBullet( launcher:shepCharacter, offset:Point, offsetDist:float, speed:float):void
        {
            var launchDir = theUtilities.GetVectorFromAngle( launcher.sprite.rotation);
            var bullet = GetFreeBullet();
            if( bullet == null )
                return;

            bullet.Init( this );
            if( bullet.sprite == null || bullet.faction != launcher.faction )
                bullet.sprite = new Image(Texture.fromAsset("assets/" + launcher.strBulletName + ".png"));
            else
            {
                trace("lol didn't need a new bullet");
                bullet.sprite.x = -100;
                bullet.sprite.y = -100;
            }
            bullet.sprite.center();
            bullet.worldPosition.x = launcher.worldPosition.x + (offset.x * offsetDist);
            bullet.worldPosition.y = launcher.worldPosition.y + (offset.y * offsetDist);

            bullet.clipRect = new Rectangle();

            bullet.faction = launcher.faction;
            bullet.sprite.rotation = launcher.sprite.rotation;

            bullet.rotationVelocity = 0;            

            //ugly!
            if( launcher.bPlayerControlled)
               bullet.moveVelocity = launcher.moveVelocity;
    
            bullet.moveVelocity.x += launchDir.x * speed;
            bullet.moveVelocity.y += launchDir.y * speed;
            bullet.remainingLifetime = 1.5;

            RegisterShepObject(bullet);
            bullet.clipRect = bullet.sprite.bounds;
            bullet.clipRect.x = bullet. worldPosition.x;
            bullet.clipRect.y = bullet.worldPosition.y;

            bullet.damage = 5;

            //trace("New bullet created! lifetime " + bullet.remainingLifetime);
            //trace("Bullet Speed! " + bullet.moveVelocity.x + " " + bullet.moveVelocity.y);
            //retFacing

            
        }

        public function LaunchEnemyBullet( launcher:shepCharacter, offset:Point, offsetDist:float, launchDir:Point, speed:float, damage:float):void
        {
            var bullet = GetFreeBullet();
            if( bullet == null )
                return;

            bullet.Init( this );
            bullet.sprite = new Image(Texture.fromAsset("assets/" + launcher.strBulletName + ".png"));
            bullet.sprite.center();
            bullet.worldPosition.x = launcher.worldPosition.x + (offset.x * offsetDist);
            bullet.worldPosition.y = launcher.worldPosition.y + (offset.y * offsetDist);
            bullet.clipRect = new Rectangle();

            bullet.faction = launcher.faction;
            bullet.sprite.rotation = launcher.sprite.rotation;
            if( launcher.strBulletName != "enemy_shot_9")
                bullet.rotationVelocity = 360 * 8;
            else
                bullet.sprite.rotation = (theUtilities.GetAngleFromVector(launchDir)/360) * 6.28;

            bullet.rotationFriction = 0.0;
    
            bullet.moveVelocity.x = launchDir.x * speed;
            bullet.moveVelocity.y = launchDir.y * speed;
            bullet.remainingLifetime = 3.0;

            RegisterShepObject(bullet);
            bullet.clipRect = bullet.sprite.bounds;
            bullet.clipRect.x = bullet. worldPosition.x;
            bullet.clipRect.y = bullet.worldPosition.y;

            bullet.damage = damage;

            //trace("New bullet created! lifetime " + bullet.remainingLifetime);
            //trace("Bullet Speed! " + bullet.moveVelocity.x + " " + bullet.moveVelocity.y);
            //retFacing

            
        }

        public function GetFreeBullet():shepBullet
        {
            for( var t:int=0; t < theCollision.projectiles.length; t++)
            {
                if( theCollision.projectiles[t].bActive == false)
                {
                    theCollision.projectiles[t].bActive = true;
                    return theCollision.projectiles[t];
                }
            }

            return null;
        }

        public function GetFreeCoin():shepCollectable
        {
            for( var t:int=0; t < theCollision.collectables.length; t++)
            {
                if( theCollision.collectables[t].bActive == false)
                {
                    theCollision.collectables[t].bActive = true;
                    return theCollision.collectables[t];
                }
            }

            return null;
        }

        public function GetFreeParticleQuad():shepParticleQuad
        {
            for( var t:int=0; t < theCollision.particleQuads.length; t++)
            {
                if( theCollision.particleQuads[t].bActive == false)
                {
                    theCollision.particleQuads[t].bActive = true;
                    return theCollision.particleQuads[t];
                }
            }

            return null;
        }


        public function SpawnQuadParticle( pos:Point, dir:Point, accel:Point, friction:float, color:Color, size:float, lifetime:float, fadeAfter:float ) : shepParticle
        {
            var part = new shepParticle();
            part.Init( this );
            part.SetColor(color);
            part.size = size;
            part.moveDir = dir;
            part.accelDir = accel;
            part.friction = friction;
            part.worldLoc = pos;
            part.lifetime = lifetime;
            part.fadeAfter = fadeAfter;

            RegisterShepObject(part);
            return part;
        }

        public function SpawnImageParticle( strImage:string, pos:Point, dir:Point, accel:Point, friction:float, color:Color, size:float, lifetime:float, fadeAfter:float, rot:float ) : shepParticle
        {
            var part = new shepParticle();
            part.bImageParticle=true;
            part.Init( this );
            part.SetImage(strImage,size);
            part.size = size;
            part.moveDir = dir;
            part.accelDir = accel;
            part.friction = friction;
            part.worldLoc = pos;
            part.lifetime = lifetime;
            part.fadeAfter = fadeAfter;
            part.rotationSpeed = rot;

            RegisterShepObject(part);
            return part;
        }

        public function SpawnCoin( pos:Point, dir:Point, value:int):void
        {
            if( gameState != 2 || playerShip == null)
                return;
            var coin = GetFreeCoin();
            if( coin == null )
            {
                playerShip.shipHealth += value;
                return;
            }
            coin.Init( this );

            if( coin.sprite == null || coin.value != value)
            {
                if( value == 5)
                    coin.sprite = new Image(Texture.fromAsset("assets/five_coin.png"));
                else
                    coin.sprite = new Image(Texture.fromAsset("assets/one_coin.png"));
            }
            else
            {
                coin.sprite.x = -100;
                coin.sprite.y = -100;
            }
            coin.sprite.center();
            coin.worldPosition = pos;
            coin.moveVelocity = dir;
            coin.bDriftTowardsPlayer = true;
            coin.alreadyCollectedGoAway = false;
            coin.bIncreasePlayerPowerLevel = false;
            coin.value = value;
            coin.fLifetime = 0.0;

            RegisterShepObject(coin);
            coin.clipRect = coin.sprite.bounds;
            coin.clipRect.x = coin.worldPosition.x;
            coin.clipRect.y = coin.worldPosition.y;
        }

        public function SpawnEnemy( etype:int, worldLoc:Point ):shepEnemy
        {
            if( gameState != 2 || playerShip == null)
                return null;

            var newEnemy:shepEnemy = null;

            switch( etype )
            {
                case 2:
                    newEnemy = new shepEnemy_Type2();
                    break;
                case 3:
                    newEnemy = new shepEnemy_Type3();
                    break;
                case 4:
                    newEnemy = new shepEnemy_Type4();
                    break;
                case 5:
                    newEnemy = new shepEnemy_Type5();
                    break;
                case 6:
                    newEnemy = new shepEnemy_Type6();
                    break;
                case 7:
                    newEnemy = new shepEnemy_Type7();
                    break;
                case 72:
                    newEnemy = new shepEnemy_Type7_2();
                    break;
                case 8:
                    newEnemy = new shepEnemy_Type8();
                    break;
                case 9:
                    newEnemy = new shepEnemy_Type9();
                    break;
            }

            newEnemy.worldPosition = worldLoc;
            newEnemy.Init( this );
            RegisterShepObject(newEnemy);
            theCollision.bodies.push(newEnemy);
            newEnemy.clipRect = newEnemy.sprite.bounds;
            newEnemy.clipRect.x = newEnemy.worldPosition.x;
            newEnemy.clipRect.y = newEnemy.worldPosition.y;

            return newEnemy;
        }
    }


}