package
{
    import loom.Application;
    import loom2d.display.StageScaleMode;
    import loom2d.display.Image;

    import loom2d.textures.Texture;
    import loom2d.ui.SimpleLabel;
    import loom2d.events.KeyboardEvent;

    import loom2d.math.Point;
    import loom2d.math.Rectangle;
    import loom2d.math.Color;

    import loom.gameframework.TimeManager;
    import loom.gameframework.ITicked;
    import loom.gameframework.IAnimated;


    import loom.platform.LoomKey;

    import loom2d.display.Quad;

    public class shepAutoLaser extends shepObject
    {
        var lazar:Quad;
        var worldRotation:float;
        var offsetFromShip:Point;
        public var fireAngleRelativeToShip:float;
        public var connectedShip:shepPlayerShip;
        var color:Color;

        var lockedTarget:shepCharacter;

        var fireTimer:float;
        var damagePulseTimer:float;
        var bIsFiring:boolean;

        public var worldPosition:Point;

        public var sprite:Image;

        public override function RegisterDrawElements( app:LD48Game ):void 
        {
            app.GetStage().addChild(lazar);
            app.GetStage().addChild(sprite);
        }
        public override function UnRegisterDrawElements( app:LD48Game ):void
        {
            app.GetStage().removeChild(lazar);
            app.GetStage().removeChild(sprite);
            lockedTarget = null;
        }

        public override function Init( app:LD48Game ): void 
        {
            lazar = new Quad(0,0,0,false);
            lockedTarget = null;
        }

        public function SetColor( _color:Color):void
        {
            color = _color;
            lazar.setVertexColor(0,color.toInt());
            lazar.setVertexColor(1,color.toInt());
            lazar.setVertexColor(2,color.toInt());
            lazar.setVertexColor(3,color.toInt());
        }

        function HideLaser():void
        {
            SetColor( new Color(0,0,0,0));
            lazar.setVertexPosition(0,0,0);
            lazar.setVertexPosition(1,0,0);
            lazar.setVertexPosition(2,0,0);
            lazar.setVertexPosition(3,0,0);

        }

        override public function onTick():void
        {
            var theGame = theUtilities.theGame;

            if( theGame.gameState != 2 || theGame.playerShip == null)
                return;


            var oldTime = lastUpdateTime;
            lastUpdateTime = theUtilities.time.virtualTime;
            deltaMS = lastUpdateTime - oldTime;
            deltaMS /= 1000.0;

            sprite.x = worldPosition.x - theGame.currentCameraPosition.x;
            sprite.y = worldPosition.y - theGame.currentCameraPosition.y;

            if( lockedTarget != null &&  lockedTarget.health <= 0)
            {
                lockedTarget = null;
                HideLaser();
            }

            if( lockedTarget == null )
            {
                SearchForTarget();
            }

            if( lockedTarget != null )
            {
                var targetVec:Point = lockedTarget.worldPosition - worldPosition;
                targetVec.normalize();
                sprite.rotation = (theUtilities.GetAngleFromVector( targetVec ) / 360) * 6.28;
            }

            fireTimer -= deltaMS;
            if( !bIsFiring && fireTimer <= 0 )
            {
                if( lockedTarget != null )  
                {
                    fireTimer = 3.0;
                    bIsFiring = true;
                }
                else
                {
                    //trace("No target! But i'll try again in 0.25");
                    fireTimer = 0.25;
                }
            }

            if( bIsFiring )
            {
                if( lockedTarget != null && lockedTarget.health > 0)
                {
                    SetColor( new Color( 192 + Math.random() * 64, Math.random() * 160,Math.random() * 160,255));
                    lazar.setVertexPosition(0,sprite.x-3, sprite.y-3);
                    lazar.setVertexPosition(1,lockedTarget.sprite.x+3, lockedTarget.sprite.y-3);
                    lazar.setVertexPosition(2,sprite.x-3, sprite.y+3);
                    lazar.setVertexPosition(3,lockedTarget.sprite.x+3, lockedTarget.sprite.y+3);

                    damagePulseTimer -= deltaMS;
                    if( damagePulseTimer <= 0 )
                    {
                        //trace("damaging target with laser");
                        damagePulseTimer = 0.5;
                        lockedTarget.TakeDamage(10);
                    }

                    if( lockedTarget.health <= 0)
                    {
                        lockedTarget = null;
                        HideLaser();
                    }
                }

                if( fireTimer <= 0.0)
                {
                    fireTimer = 3.0 + Math.random() * 1.0;
                    //trace("done shooting lasers now");
                    HideLaser();
                    bIsFiring = false;
                }

            }
            else
            {
                HideLaser();
            }

        }

        public function SearchForTarget():void
        {
            //determine our arc of fire 
            var fireDir:float = connectedShip.sprite.rotation + fireAngleRelativeToShip;

            var arcCenter:Point = theUtilities.GetVectorFromAngle(fireDir);
            arcCenter.normalize();

            var bestDist:float = 1200;
            var bestTarget:shepCharacter = null;

            for(var t:int = 0, l:int = theCollision.bodies.length;  t < l; t++)
            {
                var checkTarget:shepCharacter = theCollision.bodies[t];

                if( checkTarget == null )
                    continue;

                if( checkTarget.health <= 0 )
                    continue;

                if( checkTarget == connectedShip )
                    continue;

                if( checkTarget.faction == connectedShip.faction)
                    continue;

                //in the fire arc
                var deltaVec:Point = checkTarget.worldPosition - connectedShip.worldPosition;
                var dotDelta = theUtilities.Dot2D( deltaVec, arcCenter);
                if( dotDelta < 0.1)
                    continue;

                var dist:float = deltaVec.length;
                if( dist > bestDist)
                    continue;

                bestDist = dist;
                bestTarget = checkTarget;
            }     

            lockedTarget = bestTarget;
        }

    }
}