package
{
    import loom.Application;
    import loom2d.display.StageScaleMode;
    import loom2d.display.Image;

    import loom2d.textures.Texture;
    import loom2d.ui.SimpleLabel;
    import loom2d.events.KeyboardEvent;

    import loom2d.math.Point;
    import loom2d.math.Rectangle;
    import loom2d.math.Color;

    import loom.gameframework.TimeManager;
    import loom.gameframework.ITicked;

    import loom.platform.LoomKey;

    import loom2d.display.Quad;

    public class shepCharacter extends shepActor
    {
        public var lifebar:shepStatusBar;

        public var health:float;
        public var maxHealth:float;

        public var baseMovespeed:float;
        public var baseRotspeed:float;

        public var rotAcceleration:float;
        public var moveAcceleration:float;

        public var strBulletName:string;

        public override function Init( app:LD48Game ):void
        {
            health = 100.0;
            maxHealth = 100.0;
            baseMovespeed = 600.0;
            baseRotspeed = 360.0;
            lifebar = new shepStatusBar();
            lifebar.Init( app );
            lifebar.fullColor = new Color(0,255,0,255);
            lifebar.emptyColor = new Color(0255,0,0,255);
            lifebar.xLoc = 50;
            lifebar.yLoc = 50;
            //app.RegisterShepObject(lifebar);
        }

        public override function RegisterDrawElements( app:LD48Game ):void
        {
            super.RegisterDrawElements(app);
            if( lifebar != null )
            {
                lifebar.Width = sprite.width;
                lifebar.Height = 10;
            }
        }

        override public function onTick():void
        {
            if( bPlayerControlled )
            {
                HandlePlayerInput();
            }

            super.onTick();

            //spin!
            rotationVelocity += (rotAcceleration * 0.016);

            if( rotationVelocity > baseRotspeed )
                rotAcceleration = baseRotspeed;
            else if( rotationVelocity < (baseRotspeed * -1.0))
                rotationVelocity = (baseRotspeed * -1);

            if( moveVelocity.x > baseMovespeed * 1.25)
                moveVelocity.x = baseMovespeed * 1.25;
            if( moveVelocity.x < baseMovespeed * -1.25)
                moveVelocity.x = baseMovespeed * -1.25;
            if( moveVelocity.y > baseMovespeed * 1.25)
                moveVelocity.y = baseMovespeed * 1.25;
            if( moveVelocity.y < baseMovespeed * -1.25)
                moveVelocity.y = baseMovespeed * -1.25;

        }

        protected function HandlePlayerInput() : void
        {
            var theGame:LD48Game = theUtilities.theGame;

            moveDir.x = 0;
            moveDir.y = 0;

            rotAcceleration = 0.0;
            moveAcceleration = 0.0;

            if( theInput.Left )
                rotationVelocity = -baseRotspeed;
            else if( theInput.Right )
                rotationVelocity = baseRotspeed;
            else
                rotationVelocity = 0.0;

            var facing:Point = Point.polar(1.0, sprite.rotation);
            thrust.x = 0;
            thrust.y = 0;
            if( theInput.Up )
            {
                thrust.x += baseMovespeed * facing.y;
                thrust.y += baseMovespeed * facing.x;

                if( Math.random() < 0.4)
                    SpawnThrustParticle();
            }

            if( theInput.Down )
            {
                thrust.x += baseMovespeed * facing.y * -1.25;
                thrust.y += baseMovespeed * facing.x * -1.25;
            }

            if( theInput.GetAndClearSpace() )
            {
                LaunchAttack();
            }

        }

        public function SetThrustOffset( ang:float, dist:float ):Point
        {
            var vec:Point = theUtilities.GetVectorFromAngle(ang + sprite.rotation);
            var ret:Point = new Point( worldPosition.x + vec.x * dist, worldPosition.y + vec.y * dist);

            ret.x += ( -2 + Math.random() * 4);
            ret.y += ( -2 + Math.random() * 4);

            return ret;
        }


        public function SpawnThrustParticle():void
        {
            var theGame = theUtilities.theGame;
            var spawnDir:Point = thrust;
            spawnDir.x *= -1;
            //spawnDir.y *= -1;

            spawnDir.normalize(1);

            spawnDir.x += (-0.2 + Math.random() * 0.4);
            spawnDir.y += (-0.2 + Math.random() * 0.4);

            spawnDir.x *= 275;
            spawnDir.y *= 275;

            spawnDir.x += moveVelocity.x;
            spawnDir.y += moveVelocity.y;

            theGame.SpawnQuadParticle( SetThrustOffset(3.14159,32), spawnDir, new Point(0,0), 0, new Color(255,160,16,255), 4.0,1.0,1.0 );
        }


        public function TakeDamage( d:float ):boolean
        {
            if( health < 0)
                return false;

            var theGame:LD48Game = theUtilities.theGame;

            health -= d;
            //trace("Damaged! Health " + health);

            if( health <= 0)
            {
                OnDeath();
                theGame.UnRegisterShepObject(this);
            }

            return (health > 0);            
        }

        public function LaunchAttack():void
        {
            theUtilities.theGame.LaunchBullet( this, new Point(0,0), 0.0,500.0);
        }

        override public function CollideWithBullet( b:shepBullet ):boolean
        {
            var theGame:LD48Game = theUtilities.theGame;
            super.CollideWithBullet(b);
            if( b.faction == faction )
                return false;

            TakeDamage(b.damage);
            return true;
        }
    }

}