package
{
    import loom.Application;
    import loom2d.display.StageScaleMode;
    import loom2d.display.Image;

    import loom2d.textures.Texture;
    import loom2d.ui.SimpleLabel;
    import loom2d.math.Point;
    import loom2d.math.Rectangle;
    import loom2d.math.Color;

    import loom2d.events.KeyboardEvent;

    import loom.gameframework.TimeManager;
    import loom.gameframework.ITicked;

    import loom.platform.LoomKey;

    import loom2d.display.Quad;

    public class shepParticleQuad
    {
        public var quad:Quad;
        public var bActive:Boolean;
    }

    public class shepParticle extends shepObject
    {
        public var moveDir:Point;
        public var accelDir:Point;
        public var friction:float;

        public var worldLoc:Point;
        public var screenLoc:Point;
        public var size:float;
        public var rotationSpeed:float;


        public var lifetime:float;
        public var fadeAfter:float;

        public var particleQuad:shepParticleQuad;
        public var color:Color;
        public var bImageParticle:Boolean;
        public var image:Image;

        public override function Init( app:LD48Game ): void 
        {
            particleQuad = theUtilities.theGame.GetFreeParticleQuad();
            if( particleQuad != null && particleQuad.quad == null)
                particleQuad.quad = new Quad(0,0,0,false);
        }

        public function SetImage(strImage:string, size:float):void
        {
            bImageParticle = true;
            image = new Image(Texture.fromAsset("assets/" + strImage + ".png"));

        }

        public function SetColor( _color:Color):void
        {
            if( particleQuad == null )
                return;

            color = _color;
            particleQuad.quad.setVertexColor(0,color.toInt());
            particleQuad.quad.setVertexColor(1,color.toInt());
            particleQuad.quad.setVertexColor(2,color.toInt());
            particleQuad.quad.setVertexColor(3,color.toInt());
        }

        public override function RegisterDrawElements( app:LD48Game ):void
        {
            if( bImageParticle )
                app.GetStage().addChild(image);
            else if( particleQuad != null )
                app.GetStage().addChild(particleQuad.quad);
    
        }

        public override function UnRegisterDrawElements( app:LD48Game ):void
        {
            if( bImageParticle )
                app.GetStage().removeChild(image);
            else if( particleQuad != null )
                app.GetStage().removeChild(particleQuad.quad);

        }

        override public function onTick():void 
        {
            super.onTick();

            var oldTime = lastUpdateTime;
            lastUpdateTime = theUtilities.time.virtualTime;
            deltaMS = lastUpdateTime - oldTime;
            deltaMS /= 1000.0;

            var theGame = theUtilities.theGame;
            
            //move along
            moveDir.x -= moveDir.x * friction * deltaMS;
            moveDir.y -= moveDir.y * friction * deltaMS;
            
            moveDir.x += accelDir.x * deltaMS;
            moveDir.y += accelDir.y * deltaMS;

            worldLoc.x += moveDir.x * deltaMS;
            worldLoc.y += moveDir.y * deltaMS;

            screenLoc.x = worldLoc.x - theGame.currentCameraPosition.x;
            screenLoc.y = worldLoc.y - theGame.currentCameraPosition.y;

            //tick
            lifetime -= deltaMS;
            var alpha = 1.0;

            if( lifetime <= 0 )
            {
                theUtilities.theGame.UnRegisterShepObject(this);
                if( image != null )
                    image.dispose();

                if( particleQuad != null )
                    particleQuad.bActive = false;

                particleQuad = null;
                return;
            }
            else if( lifetime < fadeAfter)
            {
                alpha = lifetime/fadeAfter;
            }

            if( bImageParticle != true )
            {
                var hs:float = size * 0.5;

                if( particleQuad != null )
                {
                    particleQuad.quad.setVertexPosition(0,screenLoc.x - hs, screenLoc.y - hs);
                    particleQuad.quad.setVertexPosition(1,screenLoc.x + hs, screenLoc.y - hs);
                    particleQuad.quad.setVertexPosition(2,screenLoc.x - hs, screenLoc.y + hs);
                    particleQuad.quad.setVertexPosition(3,screenLoc.x + hs, screenLoc.y + hs);
                    particleQuad.quad.alpha = alpha;
                }
            }
            else
            {
                image.center();
                image.x = screenLoc.x;
                image.y = screenLoc.y;
                image.rotation += rotationSpeed * deltaMS;
                image.alpha = alpha;
            }


        }
    }
}